module Ecom
  module Core
    class SiteCrew < ApplicationRecord
      TEMPORARY_SITE_CREW = 'Temporary site crew'.freeze
      PERMANENT_SITE_CREW = 'Permanent site crew'.freeze

      SITE_CREW_TYPES = [TEMPORARY_SITE_CREW, PERMANENT_SITE_CREW].freeze

      ACTIVE = 'Active'.freeze
      INACTIVE = 'Inactive'.freeze
      STATUSES = [ACTIVE, INACTIVE].freeze

      validates :crew_id, :crew, :site_id, :site, :start_date, :site_crew_type, :status, presence: true

      validates_uniqueness_of :site_id,
                              scope: %i[crew_id status],
                              if: :active_site_crew_exists?,
                              message: 'There can only be one record with status `Active` for a given site and crew'

      validates :site_crew_type, inclusion: SITE_CREW_TYPES
      validates :status, inclusion: STATUSES

      belongs_to :crew
      belongs_to :site

      def active_site_crew_exists?
        active_site_crew = Ecom::Core::SiteCrew.where(site_id: site_id, crew_id: crew_id, status: ACTIVE)

        return true if !active_site_crew.count.zero? && status == ACTIVE

        false
      end
    end
  end
end
